<?php

namespace Drupal\advanced_file_destination\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Controller for managing directory state.
 */
class DirectoryStateController extends ControllerBase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new DirectoryStateController.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * Gets the default directory from configuration.
   *
   * @return string
   *   The configured default directory or 'public://' as fallback.
   */
  protected function getDefaultDirectory() {
    return $this->configFactory->get('advanced_file_destination.settings')->get('default_directory') ?: 'public://';
  }

  /**
   * Gets valid stream wrappers from configuration.
   *
   * @return array
   *   Array of valid stream wrapper schemes.
   */
  protected function getValidStreamWrappers() {
    $config = $this->configFactory->get('advanced_file_destination.settings');
    $wrappers = [];
    
    // Add all configured stream wrappers
    if ($config->get('include_public')) {
      $wrappers[] = 'public';
    }
    if ($config->get('include_private')) {
      $wrappers[] = 'private';
    }
    if ($config->get('include_assets')) {
      $wrappers[] = 'assets';
    }
    if ($config->get('include_temporary')) {
      $wrappers[] = 'temporary';
    }
    
    // If no wrappers are configured, default to public and private
    if (empty($wrappers)) {
      $wrappers = ['public', 'private'];
    }
    
    return $wrappers;
  }

  /**
   * Normalizes a directory path.
   */
  protected function normalizeDirectoryPath($path) {
    if (empty($path)) {
      return $this->getDefaultDirectory();
    }

    // Get valid stream wrappers
    $valid_wrappers = $this->getValidStreamWrappers();
    $valid_wrappers_regex = implode('|', $valid_wrappers);
    
    // If already has a valid scheme, return as is
    if (preg_match('#^(' . $valid_wrappers_regex . ')://#', $path)) {
      // Remove any duplicate scheme
      $path = preg_replace('#^(' . $valid_wrappers_regex . ')://((' . $valid_wrappers_regex . ')://)#', '$1://', $path);
      return $path;
    }

    // Get the default directory scheme
    $default_dir = $this->getDefaultDirectory();
    $default_scheme = 'public://';
    
    // Extract scheme from default directory
    if (preg_match('#^([a-z]+)://#', $default_dir, $matches)) {
      $default_scheme = $matches[1] . '://';
    }

    // Add scheme if missing
    return $default_scheme . ltrim($path, '/');
  }

  /**
   * Updates directory selection state.
   */
  public function updateState(Request $request) {
    $directory = $request->request->get('directory');
    
    if (empty($directory)) {
      return new JsonResponse(['status' => 'error', 'message' => 'No directory provided'], 400);
    }

    try {
      // Always normalize directory path
      $directory = $this->normalizeDirectoryPath($directory);
      
      // Get current directory from manager
      $manager = \Drupal::service('advanced_file_destination.manager');
      $current = $manager->getDestinationDirectory();

      // Only update if directory has changed
      if ($current !== $directory) {
        if (!$manager->setDestinationDirectory($directory)) {
          throw new \Exception('Failed to set directory');
        }

        \Drupal::logger('advanced_file_destination')->notice(
          'Directory explicitly set: @dir (Previous: @prev)', 
          ['@dir' => $directory, '@prev' => $current]
        );
      }

      return new JsonResponse([
        'status' => 'success',
        'directory' => $directory,
        'previous' => $current
      ]);
    }
    catch (\Exception $e) {
      return new JsonResponse([
        'status' => 'error',
        'message' => $e->getMessage()
      ], 500);
    }
  }
}
